// The theme is ALONE
#include <stdio.h>
#include <stdlib.h>
#include <cstring>

#include "SDL/SDL.h"
#include "SDL/SDL_Mixer.h"
#include "SDL/SDL_Image.h"

#include "gfx.h"

#define TRUE -1
#define FALSE 0

#define BUFWIDTH 320
#define BUFHEIGHT 240

#define NUMSTARS 100
#define STARSHIFT 3

#define MAXUFOS 100

#define MAXSHOTS 10
#define MAXPARTICLES 10000

Mix_Chunk *explode = NULL;
Mix_Chunk *blung = NULL;
Mix_Chunk *zap = NULL;
int wave=0;

typedef struct tparticle {
	int state;
	float x;
	float y;
	float dx;
	float dy;
};

tparticle particles[MAXPARTICLES];

typedef struct tufo {
	float x;
	float y;
	int hp;
	int state;
	int lframe;
	float speed;
};
tufo ufos[MAXUFOS];

typedef struct tshot {
	int state;
	float x;
	float y;
	float dx;
	float dy;
};
tshot shots[MAXSHOTS];
int cshotmax=5;

int stars[NUMSTARS * 2];

#define LEVEL_WIDTH 20
#define LEVEL_HEIGHT 90
char level[LEVEL_WIDTH][LEVEL_HEIGHT];

char *buffer = NULL;
SDL_Surface *screen;
Mix_Music *music = NULL;
Mix_Music *mining = NULL;

int silos[6];

//For framerate limiting
Uint32 to=0;
Uint32 cframe=0;

void copyBuffer() {
	int x, y;
	int size = screen->w / BUFWIDTH;
	SDL_Rect dst;
	Uint8 r; Uint8 g; Uint8 b;
	dst.w=size;
	dst.h=size;
	dst.y=0;

	for(y=0; y<BUFHEIGHT; y++) {
		dst.x = 0;
		for(x=0; x<BUFWIDTH; x++) {
#if SDL_BYTEORDER == SDL_LIL_ENDIAN
			b = buffer[((y*BUFWIDTH)+x)*4];
			g = buffer[(((y*BUFWIDTH)+x)*4)+1];
			r = buffer[(((y*BUFWIDTH)+x)*4)+2];
#else
			r = buffer[((y*320)+x)*4];
			g = buffer[(((y*320)+x)*4)+1];
			b = buffer[(((y*320)+x)*4)+2];
#endif

			SDL_FillRect(screen, &dst, SDL_MapRGB(screen->format, r, g, b));
			dst.x+=size;
		}
		dst.y+=size;
	}
	
	SDL_Flip(screen);
}

typedef struct MA_Image_t {
	int width;
	int height;
	int hasalpha;
	char *pixels;
} *MA_Image;

typedef struct managedmem {
	Uint16 type;
	char *name;
	void * mem;
	managedmem *next;
	managedmem *prev;
} *mmem;

mmem head = NULL;

mmem allocM(size_t Size, char *name) {
	void * t;
	if(Size!= 0) {
		t = malloc(Size);
	} else {
		t = NULL;
	}

	mmem ptr = (mmem)malloc(sizeof(managedmem));
	ptr->type = 1;
	ptr->name = name;
	ptr->mem = t;
	ptr->next = NULL;

	if(head==NULL) {
		ptr->prev = NULL;
		head = ptr;
	} else {
		mmem tptr = head;
		while(tptr->next!=NULL) {
			tptr = tptr->next;
		}
		ptr->prev = tptr;
		tptr->next = ptr;
	}
	
	return ptr;
}

void freeAllocs() {
 	mmem t = head;
	while(head!=NULL) {
		if(head->next == NULL) {
			if(head->mem != NULL) free(head->mem);
			free(head);
			head = NULL;
		} else {
			if(t->next == NULL) {
				if(t!=head) {
					if(t->mem!=NULL) free(t->mem);
					t=t->prev;
					free(t->next);
					t->next=NULL;
					t=head;
				}
			} else {
				t = t->next;
			}
		}
	}
}

mmem MA_LoadImage(char *filename, char *objname) {
	SDL_Surface *surf = IMG_Load(filename);
	if(!surf) printf(SDL_GetError());
	MA_Image m = (MA_Image)malloc(sizeof(MA_Image_t));
	m->height = surf->h;
	m->width = surf->w;
	if(surf->format->Amask != 0) m->hasalpha = SDL_TRUE;
	m->pixels = (char *)malloc(surf->w*surf->h*4);

	Uint32 p; Uint8 r, g, b, a;
	for(int y=0; y<surf->h; y++) {
		for(int x=0; x<surf->w; x++) {
			p = SDL_getpixel(surf, x, y);
			SDL_GetRGBA(p, surf->format, &r, &g, &b, &a);
			m->pixels[((y*surf->w) + x)*4] = a;
			m->pixels[(((y*surf->w) + x)*4)+1] = b;
			m->pixels[(((y*surf->w) + x)*4)+2] = g;
			m->pixels[(((y*surf->w) + x)*4)+3] = r;
		}
	}

	mmem ma = allocM(0, objname);
	ma->mem = m;

	SDL_FreeSurface(surf);

	return ma;
}

void MA_MakeTransparent(mmem obj) {
	MA_Image m = (MA_Image)obj->mem;
	m->hasalpha=SDL_TRUE;

	for(int y=0; y<m->height; y++) {
		for(int x=0; x<m->width; x++) {
			if(((Uint8)m->pixels[((y*m->width+x)*4)+1] == 0xff)) {
				if(((Uint8)m->pixels[((y*m->width+x)*4)+3] == 0xff)) {
					if(((Uint8)m->pixels[((y*m->width+x)*4)+2] == 0)) {
						m->pixels[((y*m->width+x)*4)] = 0;
					}
				}
			}
		}
	}
}


void BlitSubImg(mmem img, int dx, int dy, int x, int y, int w, int h) {
	MA_Image m = (MA_Image)img->mem;

	for(int cy=0; cy<h; cy++) {
		while((cy+dy)<0) cy++;
		if((cy+dy) >= BUFHEIGHT) return;
		for(int cx=0; cx<w; cx++) {
			while((cx+dx)<0) cx++;
			if(!((cx+dx) >= BUFWIDTH)) {
				if(m->pixels[(((cy*m->width) + cx) * 4)]) {
					buffer[((((dy+cy)*BUFWIDTH) + dx + cx)*4)+0] = m->pixels[((((cy+y)*m->width) + cx+x) * 4)+1];
					buffer[((((dy+cy)*BUFWIDTH) + dx + cx)*4)+1] = m->pixels[((((cy+y)*m->width) + cx+x) * 4)+2];
					buffer[((((dy+cy)*BUFWIDTH) + dx + cx)*4)+2] = m->pixels[((((cy+y)*m->width) + cx+x) * 4)+3];
				}
			}
		}
	}
}

void BlitImg(mmem img, int x, int y) {
	MA_Image m = (MA_Image)img->mem;

	if((y+m->height)<0) return;

	for(int cy=0; cy<m->height; cy++) {
		while((cy+y)<0) cy++;
		if((cy+y) >= BUFHEIGHT) return;
		if(x+m->width>=0) {
			for(int cx=0; cx<m->width; cx++) {
				while((cx+x)<0) cx++;
				if(!((cx+x) >= BUFWIDTH)) {
					if(m->pixels[(((cy*m->width) + cx) * 4)]) {
						buffer[((((y+cy)*BUFWIDTH) + x + cx)*4)+0] = m->pixels[(((cy*m->width) + cx) * 4)+1];
						buffer[((((y+cy)*BUFWIDTH) + x + cx)*4)+1] = m->pixels[(((cy*m->width) + cx) * 4)+2];
						buffer[((((y+cy)*BUFWIDTH) + x + cx)*4)+2] = m->pixels[(((cy*m->width) + cx) * 4)+3];
					}
				}
			}
		}
	}
}

void FillRect(int x, int y, int w, int h, Uint8 r, Uint8 g, Uint8 b) {
	if(x>BUFWIDTH) return;
	if(y>BUFHEIGHT) return;
	if((x+w)<0) return;
	if((y+h)<0) return;

	int sx = x; if(sx < 0) sx = 0;
	int sy = y; if(sy < 0) sy = 0;

	int ex = x + w; if(ex > BUFWIDTH) ex = BUFWIDTH;
	int ey = y + h; if(ey > BUFHEIGHT) ey = BUFHEIGHT;

	for(int cy = sy; cy < ey; cy++) {
		for(int cx = sx; cx < ex; cx++) {
			buffer[(((cy * BUFWIDTH) + cx)*4)] = b;
			buffer[(((cy * BUFWIDTH) + cx)*4)+1] = g;
			buffer[(((cy * BUFWIDTH) + cx)*4)+2] = r;
		}
	}


}

mmem getObj(char *name) {
	mmem rv = head;
	while(strcmp(name, rv->name)!=0) {
		if(rv->next==NULL) return NULL;
		rv = rv->next;
	}
	return rv;
}

int noShips() {
	for(int i=0; i<MAXUFOS; i++) {
		if(ufos[i].state!=0) return(SDL_FALSE);
	}
	return(SDL_TRUE);
}

int noSilos() {
	for(int i=0; i<6; i++) {
		if(silos[i]) return(SDL_FALSE);
	}
	return(SDL_TRUE);
}

int init() {
	//Start SDL
	if(SDL_Init( SDL_INIT_EVERYTHING ) == TRUE ) { 
		printf("SDL_Init: %s\n", SDL_GetError());
		return SDL_FALSE;
	} else {
		if(Mix_OpenAudio(22050, AUDIO_S16, 2, 4096)) {
			printf("Unable to open audio!\n");
			exit(1);
		} else {
			music = Mix_LoadMUS("gfx/pewpewing.mp3");
			mining = Mix_LoadMUS("gfx/menu.mp3");
		}

		SDL_WM_SetCaption("OUTPOST ALONE", NULL);
		screen = SDL_SetVideoMode(640, 480, 0, 0);
		
		if(!screen) {
			fprintf(stderr, "Critical Error!\n");
			fprintf(stderr, "Could'nt Set Video Mode: %s\n", SDL_GetError());
			return SDL_FALSE;
		} else {
			allocM(sizeof(char) * 320 * 240 * 4, "buffer");
			//buffer = (char *)head->mem;
			buffer = (char *)getObj("buffer")->mem;
			MA_MakeTransparent(MA_LoadImage("gfx/ufo.png", "ufo"));
			MA_LoadImage("gfx/lunar.png", "dirt");
			MA_LoadImage("gfx/lunard.png", "dug");
			MA_MakeTransparent(MA_LoadImage("gfx/crosshair.png", "crosshair"));
			MA_MakeTransparent(MA_LoadImage("gfx/shot.png", "shot"));
			MA_MakeTransparent(MA_LoadImage("gfx/base.png", "base"));
			MA_MakeTransparent(MA_LoadImage("gfx/silo.png", "silo"));
			MA_MakeTransparent(MA_LoadImage("gfx/gameover.png", "gameover"));

			explode = Mix_LoadWAV("gfx/explode.wav");
			blung = Mix_LoadWAV("gfx/blung.wav");
			zap = Mix_LoadWAV("gfx/zap.wav");
			//Init the stars
			srand(SDL_GetTicks());
			for(int i=0; i<NUMSTARS; i++) {
				stars[i*2] = (rand()%BUFWIDTH)<<STARSHIFT;
				stars[i*2+1] = (rand()%BUFHEIGHT);
			}
			for(int i=0; i<MAXSHOTS; i++) {
				shots[i].state = 0;
			}
			for(int i=0; i<MAXPARTICLES; i++) {
				particles[i].state = 0;
			}
			for(int i=0; i<MAXUFOS; i++) {
				ufos[i].state=0;
			}

			for(int i=0; i<6; i++) {
				silos[i] = 1;
			}
		}
	}
	return SDL_TRUE;
}

void cleanup() {
	Mix_FreeMusic(music);
	Mix_CloseAudio();
	SDL_Quit();
	freeAllocs();
}

void clearbuffer(Uint8 r, Uint8 g, Uint8 b) {
	for(int y=0; y<240; y++) {
		for(int x=0; x<320; x++) {
#if SDL_BYTEORDER == SDL_LIL_ENDIAN
			buffer[(y*1280)+(x*4)] = b; 
			buffer[(y*1280)+(x*4)+1] = g; 
			buffer[(y*1280)+(x*4)+2] = r; 
#else
			buffer[(y*1280)+(x*4)] = r; 
			buffer[(y*1280)+(x*4)+1] = g; 
			buffer[(y*1280)+(x*4)+2] = b; 
#endif
		}
	}
}

int checkCollision(int x1, int y1, int w1, int h1, MA_Image i1, int x2, int y2, int w2, int h2, MA_Image i2) {
	if((x1 < x2) && (x1+w1 < x2)) return SDL_FALSE;
	if((x1 > (x2+w2)) && (x1+w1 > x2+w2)) return SDL_FALSE;
	if((y1 < y2) && (y1+h1 < y2)) return SDL_FALSE;
	if((y1 > (y2+h2)) && (y1+h1 > y2+h2)) return SDL_FALSE;

	// If we get here then there has to be some kind of overlap, get the shared bounds
	int ox, oy, ow, oh;

	if(x1>x2) ox = x1; else ox = x2;
	if(y1>y2) oy = y1; else oy = y2;
	if((x1+w1)<(x2+w2)) ow = (x1+w1)-ox; else ow = (x2+w2)-ox;
	if((y1+h1)<(y2+h2)) oh = (y1+h1)-oy; else oh = (y2+h2)-oy;

	for(int y=oy; y<oy+oh; y++) {
		for(int x=ox; x<ox+ow; x++) {
			if((Uint8)i1->pixels[((y-y1) * i1->width + (x-x1)) * 4] && (Uint8)i2->pixels[((y-y2) * i2->width + (x-x2)) * 4]) return SDL_TRUE;
		}
	}
	
	return SDL_FALSE;
}

void movestars() {
	for(int i=0; i<NUMSTARS; i++) {
		stars[i*2] += (i%4)+1;
		if((stars[i*2]>>STARSHIFT) > BUFWIDTH) stars[i*2] = 0;
	}
}

void drawstars() {
	Uint8 r, g, b;
	for(int i=0; i<NUMSTARS; i++) {
		if((i%4)==3) {r = 0xff; g = 0xff; b = 0xff;};
		if((i%4)==2) {r = 0xC0; g = 0xC0; b = 0xC0;};
		if((i%4)==1) {r = 0x80; g = 0x80; b = 0x80;};
		if((i%4)==0) {r = 0x40; g = 0x40; b = 0x40;};
		FillRect(stars[i*2]>>STARSHIFT, stars[i*2+1], 1, 1, r, g, b);
	}
}

void initLevel() {
	for(int y=0; y<LEVEL_HEIGHT; y++) {
		for(int x=0; x<LEVEL_WIDTH; x++) {
			level[x][y] = 0; // No tile
			if(y>11) level[x][y] = 1; //ground/dirt
		}
	}
	//level[9][12]=2;
	//level[10][12]=2;
}

void drawlevel(int cam_y) {
	mmem g = getObj("dirt");
	mmem d = getObj("dug");
	for(int y=0; y<LEVEL_HEIGHT; y++) {
		for(int x=0; x<LEVEL_WIDTH; x++) {
			switch(level[x][y]) {
			case(1):
				BlitImg(g, x*16, y*16);
				break;
			case(2):
				BlitImg(d, x*16, y*16);
				break;
			default:
				break;
			}
		}
	}
}

void initUFOS() {
	for(int i=0; i<5+wave; i++) {
		if(i==MAXUFOS) return;
		ufos[i].state = 1;
		ufos[i].y=rand()%(BUFHEIGHT/3);
		ufos[i].x=-rand()%BUFWIDTH-64;
		ufos[i].hp=1;
		ufos[i].lframe=0;
		ufos[i].speed=((rand()%4)+2)/4.0;
		ufos[i].speed+=(wave/4.0);
	}
}

void drawUFOS() {
	mmem u = getObj("ufo");
	for(int i=0; i<10; i++) {
		if(ufos[i].state!=0) BlitImg(u, (int)ufos[i].x, (int)ufos[i].y);
	}
}

void blowSilo();

void moveUFOS() {
	for(int i=0; i<10; i++) {
		if(ufos[i].state>0) {
			ufos[i].x+=ufos[i].speed;
			if(ufos[i].x>BUFWIDTH+20) {
				ufos[i].state=0;
				blowSilo();
			}
		}
	}
}

void spawnShot(int x, int y, int tx, int ty) {
	for(int i=0; i<cshotmax; i++) {
		if(shots[i].state==0) {
			Mix_PlayChannel(-1, zap, 0);
			shots[i].state = 1;
			shots[i].x=x;
			shots[i].y=y;

			if(abs(tx-x) > abs(ty-y)) {
				shots[i].dx = 1;
				shots[i].dy = (float)abs(ty-y)/(float)abs(tx-x);
			} else {
				shots[i].dy=1;
				shots[i].dx = (float)abs(tx-x)/(float)abs(ty-y);
			}
			if(tx<x) shots[i].dx = shots[i].dx * -1;
			if(ty<y) shots[i].dy = shots[i].dy * -1;
			i=MAXSHOTS;
		}
	}
}

void drawShots() {
	mmem s = getObj("shot");
	for(int i=0; i<MAXSHOTS; i++) {
		if(shots[i].state==1) {
			BlitImg(s, shots[i].x-3, shots[i].y-3);
		}
	}
}

void moveShots() {
	for(int i=0; i<MAXSHOTS; i++) {
		if(shots[i].state==1) {
			shots[i].x+=shots[i].dx;
			shots[i].y+=shots[i].dy;

			if(shots[i].x<-10) shots[i].state=0;
			if(shots[i].x>BUFWIDTH+10) shots[i].state=0;

			if(shots[i].y<-10) shots[i].state=0;
			if(shots[i].y>BUFHEIGHT+10) shots[i].state=0;
		}
	}
}

void makeExplosion(int x, int y);

void checkCollissions() {
	mmem ufo = getObj("ufo");
	mmem shot = getObj("shot");

	//UFO/shot collissions
	for(int u=0; u<MAXUFOS; u++) {
		for(int s=0; s<MAXSHOTS; s++) {
			if(shots[s].state!=0&&ufos[u].state!=0) {
				if(checkCollision(shots[s].x-3, shots[s].y-3, 7, 7, (MA_Image)shot->mem, (int)ufos[u].x, (int)ufos[u].y, 32, 16, (MA_Image)ufo->mem)) {
					Mix_PlayChannel(-1, explode, 0);
					shots[s].state=0;
					ufos[u].state=0;
					makeExplosion(((int)ufos[u].x)+16, (int)ufos[u].y+8);
				}
			}
		}
	}
}

void drawBase() {
	mmem base = getObj("base");
	mmem silo = getObj("silo");

	BlitImg(base, 160-16, 240-32-48);
	if(silos[0]) BlitImg(silo, 36-8, 240-48-32);
	if(silos[1]) BlitImg(silo, 108-8, 240-48-32);
	if(silos[2]) BlitImg(silo, 212-8, 240-48-32);
	if(silos[3]) BlitImg(silo, 276, 240-48-32);
	if(silos[4]) BlitImg(silo, 72-8, 240-48-32);
	if(silos[5]) BlitImg(silo, 244-4, 240-48-32);
}

void moveParticles() {
	for(int i=0; i<MAXPARTICLES; i++) {
		if(particles[i].state!=0) {
			particles[i].x+=particles[i].dx;
			particles[i].y+=particles[i].dy;
			particles[i].state--;
		}
	}
}

void drawParticles() {
	for(int i=0; i<MAXPARTICLES; i++) {
		if(particles[i].state!=0) {
			FillRect((int)particles[i].x, (int)particles[i].y, 1, 1, 0xff, 0, 0);
		}
	}
}

void makeExplosion(int x, int y) {
	int mp = 200;
	for(int i=0; i<MAXPARTICLES; i++) {
		if(particles[i].state==0) {
			particles[i].state=32 + rand()%16;
			particles[i].x=x;
			particles[i].y=y;
			//particles[i].dx = 1.0/((float)(rand()%20-10));
			//particles[i].dy = 1.0/((float)(rand()%20-10));

			particles[i].dx = ((float)(rand()%20-10)) / 10.0;
			particles[i].dy = ((float)(rand()%20-10)) / 10.0;
			mp--;
		}
		if(mp<=0) i=MAXPARTICLES+1;
	}
}

void blowSilo() {
	for(int i=0; i<6; i++) {
		if(silos[i]) {
			silos[i] = 0;
			Mix_PlayChannel(-1, explode, 0);
			switch(i) {
				case(0): makeExplosion(36,240-48); break;
				case(1): makeExplosion(108,240-48); break;
				case(2): makeExplosion(212,240-48); break;
				case(3): makeExplosion(276+8,240-48); break;
				case(4): makeExplosion(72,240-48); break;
				default: makeExplosion(248,240-48); break;
			}
			return;
		}
	}
}

int main( int argc, char* args[] ) { 
	if(init()) {
		SDL_ShowCursor(SDL_FALSE);
		clearbuffer(0, 0, 0);
		int done = 0;
		Uint32 t = SDL_GetTicks();
		mmem ufo = getObj("ufo");
		mmem ch = getObj("crosshair");
		mmem go = getObj("gameover");
		SDL_Event e;
		int mx, my;
		initLevel();
		//initUFOS();
		//Mix_PlayMusic(music, -1);
		to = SDL_GetTicks();
		int spawnshot=SDL_FALSE;

		Uint32 wait = SDL_GetTicks() + 30000;
		int countdown = 0;
		Uint32 lastcount = 0;

		int gamestate=0;
		//Mix_FadeOutMusic(10000);
		while(!done) {
			switch(gamestate) {
				case(0): //starting case, proceed to wait for 30
					gamestate = 1;
					wait = SDL_GetTicks() + 10000;
					Mix_FadeInMusic(mining, -1, 1000);
					break;
				case(1): //wait
					if(SDL_GetTicks() > wait) {
						gamestate = 2;
						Mix_FadeOutMusic(1000);
						countdown=3;
						lastcount = SDL_GetTicks()+1250;
					}
					break;
				case(2): // Alarm
					if(SDL_GetTicks()>lastcount) {
						countdown--;
						Mix_PlayChannel(-1, blung, 0);
						lastcount = SDL_GetTicks()+1250;
					}
					if(countdown==0) gamestate=3;
					break;
				case(3): //spawn ufos
					Mix_FadeInMusic(music, -1, 1000);
					initUFOS();
					gamestate = 4;
					break;
				case(4):
					if(noShips()) {
						countdown=2;
						lastcount = SDL_GetTicks()+1000;
						Mix_FadeOutMusic(1000);
						gamestate=5;
						wave++;
					}
					break;
				case(5): // post wave cooldown
					if(SDL_GetTicks()>lastcount) {
						countdown--;
						lastcount = SDL_GetTicks()+1000;
					}
					if(countdown==0) gamestate=0;
					break;
				case(99): //gameover

					
					break;
				default:
					break;
			}

			if(noSilos()) gamestate=99;
			spawnshot = SDL_FALSE;
			Uint32 tt = SDL_GetTicks();
			while(to<tt) {
				cframe++;
				to+=40;
			}

			while(SDL_PollEvent(&e)) {
				if(e.type==SDL_QUIT) {
					done=1;
				}
				if(e.type==SDL_KEYUP) {
					if(e.key.keysym.sym == SDLK_ESCAPE) {
						done=1;
					}
				}
				if(e.type==SDL_MOUSEMOTION) {

				}
				if(e.type==SDL_MOUSEBUTTONUP) {
					if(e.button.button==SDL_BUTTON_LEFT) {
						spawnshot = SDL_TRUE;
						// put code here to start over
					}
				}
			}
			SDL_GetMouseState(&mx, &my);
			
			clearbuffer(0,0,0);
			movestars();
			moveUFOS();
			moveShots();
			moveParticles();
			drawstars();
			drawlevel(0);
			if(spawnshot) {
				spawnShot(160, 240-48-32, (mx/2), (my/2));
			}
			checkCollissions();
			drawUFOS();
			drawShots();
			//BlitImg(gfx, 0, 0);
			/*BlitSubImg(gfx, 0,0,0,0,16,16);//floor
			BlitSubImg(gfx, 16,0,16,0,16,16);//ul corner
			BlitSubImg(gfx, 32,0,32,0,16,16);//upper wall
			BlitSubImg(gfx, 48,0,32,0,16,16);//upper wall
			BlitSubImg(gfx, 32,16,0,0,16,16);//floor
			BlitSubImg(gfx, 48,16,0,0,16,16);//floor
			BlitSubImg(gfx, 64,0,48,0,16,16);//ur corner
			BlitImg(thing, mx/2, my/2);
			BlitImg(thing, 152,112);
			checkCollision(152, 112, 16, 16, (MA_Image)thing->mem, mx/2, my/2, 16, 16, (MA_Image)thing->mem);*/
			drawParticles();
			if(gamestate!=99) {
				drawBase();
				BlitImg(ch, mx/2-7,my/2-7);
			} else {
				BlitImg(go, 0,0);
			}


			copyBuffer();
			SDL_Delay(10);
		}
	}

	cleanup();
	
	//getchar();
	return 0; 
} 